﻿module Properties

//====================================================
// Подробное описание программного кода исходника на:
// https://www.interestprograms.ru
// Исходные коды программ и игр
//====================================================


type ExplicitProperties() =
    // Закрытая переменная для хранения значения
    // свойства Simple
    let mutable simple = ""

    // Резервное хранилище
    let mutable backupStorage: int = 0

    // Минимально простое свойство класса.
    member x.Simple
        with get (): string = simple
        and set v = simple <- v

    member x.MyProperty
        with get () = backupStorage
        and set v =
            if v < 1000 then
                backupStorage <- v
            else
                backupStorage <- 1000


type AutoProperties() =
    // Автоматическое свойство инициализируется однократно
    // при создании экземпляра класса.
    // Инициализируется в первичном конструкторе.
    // При инициализации получает автоматическое резервное хранилище.
    member val SimpleAutoProperty = System.DateTime.Now with get, set
    // Авто-свойство только для чтения.
    //member val SimpleAutoProperty = System.DateTime.Now with get, set

    // Явное свойство будет принимать новые значения при
    // каждом вызове.
    member this.SimpleExplicitProperty = System.DateTime.Now

    // Инициализация автосвойства во
    // вторичном конструкторе.
    new(d: int) as con2 =
        AutoProperties()
        then con2.SimpleAutoProperty <- System.DateTime.Today.AddDays(d)


type StaticProperties() =
    let mutable localvalue: double = 123.456
    static let mutable staticvalue: double = 123.456
    static let staticImmutableValue: double = 3.14

    static member StaticProperty
        with get () = staticvalue
        and set value =
            // К локальному значению из статического свойства
            // доступ невозможен.
            //localvalue <- value
            staticvalue <- value

    //static member StaticReadOnly with get() = staticImmutableValue
    static member StaticReadOnly = staticImmutableValue

    static member val StaticAutoProp =
        let now = System.DateTime.Now
        System.DateTime.DaysInMonth(now.Year, now.Month)
